package com.example.demo.movies.service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.AlreadyExistsException;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.repository.MovieRepository;

@Service
public class MovieService {
    private final MovieRepository repository;

    public MovieService(MovieRepository repository) {
        this.repository = repository;
    }

    private void checkTitleUnique(String title) {
        if (repository.findByTitleIgnoreCase(title).isPresent()) {
            throw new AlreadyExistsException();
        }
    }

    @Transactional(readOnly = true)
    public List<MovieEntity> getAll(Long typeId, Long ageRatingId) {
        if (typeId <= 0L && ageRatingId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        } else if (typeId <= 0L) {
            return repository.findByAgeRatingId(ageRatingId);
        } else if (ageRatingId <= 0L) {
            return repository.findByTypeId(typeId);
        }

        return repository.findByTypeIdAndAgeRatingId(typeId, ageRatingId);
    }

    @Transactional(readOnly = true)
    public Page<MovieEntity> getAll(Long typeId, Long ageRatingId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);

        if (typeId <= 0L && ageRatingId <= 0L) {
            return repository.findAll(pageRequest);
        } else if (typeId <= 0L) {
            return repository.findByAgeRatingId(ageRatingId, pageRequest);
        } else if (ageRatingId <= 0L) {
            return repository.findByTypeId(typeId, pageRequest);
        }

        return repository.findByTypeIdAndAgeRatingId(typeId, ageRatingId, pageRequest);
    }

    @Transactional(readOnly = true)
    public MovieEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(MovieEntity.class, id));
    }

    @Transactional
    public MovieEntity create(MovieEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }

        checkTitleUnique(entity.getTitle());
        return repository.save(entity);
    }

    @Transactional
    public MovieEntity update(Long id, MovieEntity entity) {
        final MovieEntity existsEntity = get(id);

        existsEntity.setTitle(entity.getTitle());
        existsEntity.setType(entity.getType());
        existsEntity.setRequiresSubscription(entity.getRequiresSubscription());
        existsEntity.setPoster(entity.getPoster());
        existsEntity.setDescription(entity.getDescription());
        existsEntity.setReleaseDate(entity.getReleaseDate());
        existsEntity.setCountry(entity.getCountry());
        existsEntity.setTagline(entity.getTagline());
        existsEntity.setDirector(entity.getDirector());
        existsEntity.setAgeRating(entity.getAgeRating());
        existsEntity.setVideo(entity.getVideo());

        return repository.save(existsEntity);
    }

    @Transactional
    public MovieEntity delete(Long id) {
        final MovieEntity existsEntity = get(id);
        repository.delete(existsEntity);

        return existsEntity;
    }

    @Transactional
    public void deleteAll() {
        repository.deleteAll();
    }

    @Transactional
    public Double getAverageRating(Long id) {
        final MovieEntity existsEntity = get(id);
        Double rating = repository.getAverageRating(existsEntity.getId());

        if (rating == null) {
            return 0D;
        }
        return rating;
    }

    @Transactional
    public List<MovieEntity> getItemsByIds(List<Long> ids) {
        List<MovieEntity> items = new ArrayList<>();
        repository.findAllById(ids).forEach(item -> items.add(item));

        return items;
    }
}
