package com.example.demo.reviews.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.AlreadyExistsException;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.reviews.model.ReviewEntity;
import com.example.demo.reviews.repository.ReviewRepository;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@Service
public class ReviewService {
    private final ReviewRepository repository;
    private final UserService userService;
    private final MovieService movieService;

    public ReviewService(ReviewRepository repository, UserService userService, MovieService movieService) {
        this.repository = repository;
        this.userService = userService;
        this.movieService = movieService;
    }

    private void checkReview(Long userId, Long movieId) {
        if (repository.findByUserIdAndMovieId(userId, movieId).size() == 1) {
            throw new AlreadyExistsException();
        }
    }

    @Transactional(readOnly = true)
    public List<ReviewEntity> getAll(Long movieId, Long userId) {
        if (movieId <= 0L && userId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        }
        if (userId <= 0L && movieId > 0L) {
            return repository.findByMovieId(movieId);
        }
        if (movieId <= 0L && userId > 0L) {
            return repository.findByUserId(userId);
        } else {
            return repository.findByUserIdAndMovieId(userId, movieId);
        }
    }

    @Transactional(readOnly = true)
    public Page<ReviewEntity> getAll(Long movieId, Long userId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);

        if (movieId <= 0L && userId <= 0L) {
            return repository.findAll(pageRequest);
        }
        if (userId <= 0L && movieId > 0L) {
            return repository.findByMovieId(movieId, pageRequest);
        }
        if (movieId <= 0L && userId > 0L) {
            return repository.findByUserId(userId, pageRequest);
        } else {
            return repository.findByUserIdAndMovieId(userId, movieId, pageRequest);
        }
    }

    @Transactional(readOnly = true)
    public ReviewEntity get(Long userId, Long id) {
        userService.get(userId);
        return repository.findOneByUserIdAndId(userId, id)
                .orElseThrow(() -> new NotFoundException(ReviewEntity.class, id));
    }

    @Transactional
    public ReviewEntity create(long userId, long movieId, ReviewEntity entity) {
        checkReview(userId, movieId);

        final UserEntity existsUser = userService.get(userId);
        final MovieEntity existsmovie = movieService.get(movieId);

        entity.setMovie(existsmovie);
        entity.setUser(existsUser);
        return repository.save(entity);
    }

    @Transactional
    public ReviewEntity update(Long userId, Long id, ReviewEntity entity) {
        final ReviewEntity existsEntity = get(userId, id);

        existsEntity.setDescription(entity.getDescription());
        existsEntity.setRate(entity.getRate());
        return repository.save(existsEntity);
    }

    @Transactional
    public ReviewEntity delete(Long userId, Long id) {
        final ReviewEntity existsEntity = get(userId, id);

        repository.delete(existsEntity);
        return existsEntity;
    }

    public void deleteAll() {
        repository.deleteAll();
    }
}
