package com.example.demo.users.api;

import java.util.Date;
import java.util.List;
import java.util.Objects;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.movies.api.MovieDto;
import com.example.demo.core.api.PageDto;
import com.example.demo.core.api.PageDtoMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.usersubscription.api.SubscriptionInfoDto;
import com.example.demo.usersubscription.model.UserSubscriptionEntity;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/user")
public class UserController {
    private final UserService userService;
    private final MovieService movieService;
    private final ModelMapper modelMapper;

    public UserController(UserService userService, MovieService movieService, ModelMapper modelMapper) {
        this.userService = userService;
        this.movieService = movieService;
        this.modelMapper = modelMapper;
    }

    private UserDto toDto(UserEntity entity) {
        return modelMapper.map(entity, UserDto.class);
    }

    private SubscriptionInfoDto toSubscriptionDto(UserSubscriptionEntity entity) {
        if (!Objects.isNull(entity)) {
            return new SubscriptionInfoDto(entity.getExpirationDate().after(new Date()), entity.getActivationDate(),
                    entity.getExpirationDate());
        } else {
            return new SubscriptionInfoDto(false, null, null);
        }
    }

    private MovieDto toMovieDto(MovieEntity entity) {
        return modelMapper.map(entity, MovieDto.class);
    }

    private UserEntity toEntity(UserDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    @GetMapping
    public PageDto<UserDto> getAll(
            @RequestParam(name = "page", defaultValue = "0") int page,
            @RequestParam(name = "size", defaultValue = Constants.DEFAULT_PAGE_SIZE) int size) {
        return PageDtoMapper.toDto(userService.getAll(page, size), this::toDto);
    }

    @GetMapping("/{id}")
    public UserDto get(@PathVariable(name = "id") Long id) {
        return toDto(userService.get(id));
    }

    @PostMapping
    public UserDto create(@RequestBody @Valid UserDto dto) {
        return toDto(userService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public UserDto update(@PathVariable(name = "id") Long id, @RequestBody UserDto dto) {
        return toDto(userService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public UserDto delete(@PathVariable(name = "id") Long id) {
        return toDto(userService.delete(id));
    }

    @GetMapping("/{id}/subscription")
    public SubscriptionInfoDto getActiveSubscription(@PathVariable(name = "id") Long id) {
        return toSubscriptionDto(userService.getActiveSubscription(id));
    }

    @PostMapping("/{id}/subscription/all")
    public List<SubscriptionInfoDto> getAllSubscriptions(@PathVariable(name = "id") Long id) {
        return userService.getUserSubscriptions(id).stream().map(this::toSubscriptionDto).toList();
    }

    @PostMapping("/{id}/subscription")
    public SubscriptionInfoDto activateSubscription(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = "subscriptionId", defaultValue = "") Long subscriptionId) {
        return toSubscriptionDto(userService.activateSubscription(id, subscriptionId));
    }

    @DeleteMapping("/{id}/subscription")
    public SubscriptionInfoDto deactivateSubscription(@PathVariable(name = "id") Long id) {
        return toSubscriptionDto(userService.deactivateSubscription(id));
    }

    @GetMapping("/{id}/watchLater")
    public List<MovieDto> getWatchLater(@PathVariable(name = "id") Long id) {
        return userService.getWatchLater(id).stream().map(this::toMovieDto).toList();
    }

    @PostMapping("/{id}/watchLater")
    public UserDto addToWatchLater(@PathVariable(name = "id") Long id,
            @RequestParam(name = "movieId", defaultValue = "") Long movieId) {
        MovieEntity movie = movieService.get(movieId);
        UserEntity user = userService.addMovieToWatchLater(id, movie);

        return toDto(user);
    }

    @DeleteMapping("/{id}/watchLater")
    public UserDto deleteFromWatchLater(@PathVariable(name = "id") Long id,
            @RequestParam(name = "movieId", defaultValue = "") Long movieId) {
        MovieEntity movie = movieService.get(movieId);
        UserEntity user = userService.deleteMovieFromWatchLater(id, movie);

        return toDto(user);
    }
}
