package com.example.demo.users.service;

import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;
import com.example.demo.usersubscription.model.UserSubscriptionEntity;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.subscriptions.model.SubscriptionEntity;
import com.example.demo.subscriptions.service.SubscriptionService;

@Service
public class UserService {
    private final UserRepository repository;
    private final SubscriptionService subscriptionService;

    public UserService(UserRepository repository, SubscriptionService subscriptionService) {
        this.repository = repository;
        this.subscriptionService = subscriptionService;
    }

    private void checkEmail(String email) {
        if (repository.findByEmailIgnoreCase(email).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("User with email %s is already exists", email));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<UserEntity> getAll(int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);
        return repository.findAll(pageRequest);
    }

    @Transactional(readOnly = true)
    public UserEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }

        checkEmail(entity.getEmail());
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public UserEntity update(long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        checkEmail(entity.getEmail());

        existsEntity.setEmail(entity.getEmail());
        existsEntity.setPassword(entity.getPassword());
        return repository.save(existsEntity);
    }

    @Transactional
    public UserEntity delete(long id) {
        final UserEntity existsEntity = get(id);
        repository.delete(existsEntity);

        return existsEntity;
    }

    @Transactional
    public void deleteAll() {
        repository.deleteAll();
    }

    @Transactional(readOnly = true)
    public List<UserSubscriptionEntity> getUserSubscriptions(Long id) {
        return get(id).getUserSubscriptions().stream().toList();
    }

    @Transactional(readOnly = true)
    public UserSubscriptionEntity getActiveSubscription(Long id) {
        List<UserSubscriptionEntity> activeSubscriptions = repository.getActiveSubscriptions(id, new Date());

        if (activeSubscriptions.size() != 1) {
            return null;
        }
        return activeSubscriptions.get(0);
    }

    @Transactional()
    public UserSubscriptionEntity activateSubscription(Long id, Long subscriptionId) {
        UserSubscriptionEntity activeSubscription = getActiveSubscription(id);
        if (!Objects.isNull(activeSubscription)) {
            return activeSubscription;
        }

        final UserEntity existsUser = get(id);
        final SubscriptionEntity subscription = subscriptionService.get(subscriptionId);

        Date now = new Date();
        Date expirationDate = Date
                .from(now.toInstant().atZone(ZoneId.systemDefault()).plus(subscription.getDuration()).toInstant());

        final UserSubscriptionEntity newUserSubscription = new UserSubscriptionEntity(existsUser, subscription, now,
                expirationDate);
        newUserSubscription.setUser(existsUser);

        repository.save(existsUser);
        return newUserSubscription;
    }

    @Transactional
    public UserSubscriptionEntity deactivateSubscription(Long id) {
        UserSubscriptionEntity subscriptionToDeactivate = getActiveSubscription(id);
        if (Objects.isNull(subscriptionToDeactivate)) {
            return null;
        }

        final UserEntity existsUser = get(id);
        subscriptionToDeactivate.setExpirationDate(new Date());
        repository.save(existsUser);
        return subscriptionToDeactivate;
    }

    @Transactional(readOnly = true)
    public List<MovieEntity> getWatchLater(long id) {
        return get(id).getWatchLater().stream().toList();
    }

    @Transactional
    public UserEntity addMovieToWatchLater(Long id, MovieEntity movie) {
        final UserEntity existsEntity = get(id);

        if (!existsEntity.getWatchLater().contains(movie)) {
            existsEntity.addMovieToWatchLater(movie);
            return repository.save(existsEntity);
        }

        return existsEntity;
    }

    @Transactional
    public UserEntity deleteMovieFromWatchLater(Long id, MovieEntity movie) {
        final UserEntity existsEntity = get(id);

        if (existsEntity.getWatchLater().contains(movie)) {
            existsEntity.deleteMovieFromWatchLater(movie);
            return repository.save(existsEntity);
        }

        return existsEntity;
    }
}
