package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.error.AlreadyExistsException;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class MovieServiceTests {
	@Autowired
	private TypeService typeService;

	@Autowired
	private AgeRatingService ageRatingService;

	@Autowired
	private MovieService movieService;

	private TypeEntity type1;
	private TypeEntity type2;
	private AgeRatingEntity ageRating1;
	private AgeRatingEntity ageRating2;

	private MovieEntity last;

	@BeforeEach
	void createData() {
		removeData();

		type1 = typeService.create(new TypeEntity("Фильм"));
		type2 = typeService.create(new TypeEntity("Сериал"));

		ageRating1 = ageRatingService.create(new AgeRatingEntity("16+"));
		ageRating2 = ageRatingService.create(new AgeRatingEntity("18+"));

		movieService.create(new MovieEntity("Бойцовский клуб", type1, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сотрудник страховой компании страдает хронической бессонницей...",
				1999, "США", "Интриги. Хаос. Мыло", "Дэвид Финчер", ageRating2,
				"https://www.youtube.com/embed/dfeUzm6KF4g?si=U2D-2WNsJzkcxlBX"));
		movieService.create(new MovieEntity("Американский психопат", type1, true,
				"src/assets/main-page/posters/AmericanPsycho.jpg",
				"Днем он ничем не отличается от окружающих, и в толпе вы не обратите на него внимания...",
				2000, "США, Канада", "Респектабельная внешность дьявола", "Мэри Хэррон", ageRating2,
				"https://www.youtube.com/embed/PpUWjff_OcM?si=e7tSm4V6qYkz406B"));
		last = movieService.create(new MovieEntity("Острые козырьки", type2, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сериал рассказывает историю восхождения одной ирландской семьи...",
				2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating1,
				"https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70"));
	}

	@AfterEach
	void removeData() {
		movieService.deleteAll();
		typeService.deleteAll();
		ageRatingService.deleteAll();
	}

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> movieService.get(0L));
	}

	@Test
	void createTest() {
		Assertions.assertEquals(3, movieService.getAll(0L, 0L).size());
		Assertions.assertEquals(2, movieService.getAll(type1.getId(), 0L).size());
		Assertions.assertEquals(last, movieService.get(last.getId()));
	}

	@Test
	void createNotUniqueTest() {
		final MovieEntity nonUniqueMovie = new MovieEntity("Острые козырьки", type2, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сериал рассказывает историю восхождения одной ирландской семьи...",
				2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating1,
				"https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70");
		Assertions.assertThrows(AlreadyExistsException.class, () -> movieService.create(nonUniqueMovie));
	}

	@Test
	void createNullableTest() {
		final MovieEntity nullableItem = new MovieEntity(null, null, false, null, null, null, null, null, null, null,
				null);
		Assertions.assertThrows(DataIntegrityViolationException.class, () -> movieService.create(nullableItem));
	}

	@Test
	void updateTest() {
		final String test = "TEST";
		final MovieEntity entity = movieService.get(last.getId());
		final String oldName = entity.getTitle();
		final MovieEntity newEntity = movieService.update(last.getId(), new MovieEntity(
				test, entity.getType(), entity.getRequiresSubscription(),
				entity.getPoster(),
				entity.getDescription(),
				entity.getReleaseDate(), entity.getCountry(), entity.getTagline(), entity.getDirector(),
				entity.getAgeRating(),
				entity.getVideo()));

		Assertions.assertEquals(3, movieService.getAll(0L, 0L).size());
		Assertions.assertEquals(newEntity, movieService.get(last.getId()));
		Assertions.assertEquals(test, newEntity.getTitle());
		Assertions.assertNotEquals(oldName, newEntity.getTitle());
	}

	@Test
	void deleteTest() {
		movieService.delete(last.getId());
		Assertions.assertEquals(2, movieService.getAll(0L, 0L).size());

		final MovieEntity newEntity = movieService.create(new MovieEntity(
				last.getTitle(), last.getType(), last.getRequiresSubscription(),
				last.getPoster(),
				last.getDescription(),
				last.getReleaseDate(), last.getCountry(), last.getTagline(), last.getDirector(),
				last.getAgeRating(),
				last.getVideo()));
		Assertions.assertEquals(3, movieService.getAll(0L, 0L).size());
		Assertions.assertNotEquals(last.getId(), newEntity.getId());
	}
}
