package com.example.demo;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.error.AlreadyExistsException;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.reviews.model.ReviewEntity;
import com.example.demo.reviews.service.ReviewService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class ReviewServiceTests {
    @Autowired
    private MovieService movieService;
    @Autowired
    private TypeService typeService;
    @Autowired
    private AgeRatingService ageRatingService;
    @Autowired
    private ReviewService reviewService;
    @Autowired
    private UserService userService;

    private MovieEntity movie1;
    private MovieEntity movie2;
    private UserEntity user1;
    private UserEntity user2;
    private ReviewEntity last;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> reviewService.get(0L, 0L));
    }

    @BeforeEach
    void createData() {
        removeData();

        var type1 = typeService.create(new TypeEntity("Фильм"));
        var type2 = typeService.create(new TypeEntity("Сериал"));

        var ageRating1 = ageRatingService.create(new AgeRatingEntity("16+"));
        var ageRating2 = ageRatingService.create(new AgeRatingEntity("18+"));

        movie1 = movieService.create(new MovieEntity("Американский психопат", type1, true,
                "src/assets/main-page/posters/AmericanPsycho.jpg",
                "Днем он ничем не отличается от окружающих, и в толпе вы не обратите на него внимания...",
                2000, "США, Канада", "Респектабельная внешность дьявола", "Мэри Хэррон", ageRating2,
                "https://www.youtube.com/embed/PpUWjff_OcM?si=e7tSm4V6qYkz406B"));
        movie2 = movieService.create(new MovieEntity("Острые козырьки", type2, false,
                "src/assets/main-page/posters/PeakyBlinders.png",
                "Сериал рассказывает историю восхождения одной ирландской семьи...",
                2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating1,
                "https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70"));

        user1 = userService.create(new UserEntity("myemail@mail.com", "123"));
        user2 = userService.create(new UserEntity("secondemail@mail.com", "321"));

        reviewService.create(user1.getId(), movie1.getId(), new ReviewEntity("dsdd", 5));
        reviewService.create(user2.getId(), movie1.getId(), new ReviewEntity("dggfg", 4));
        last = reviewService.create(user1.getId(), movie2.getId(), new ReviewEntity("dsddddd", 5));
    }

    @AfterEach
    void removeData() {
        reviewService.deleteAll();
        movieService.deleteAll();
        typeService.deleteAll();
        ageRatingService.deleteAll();
        userService.deleteAll();
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, reviewService.getAll(0L, 0L).size());
        Assertions.assertEquals(1, reviewService.getAll(movie1.getId(), user1.getId()).size());
        Assertions.assertEquals(last, reviewService.get(user1.getId(), last.getId()));
        Assertions.assertThrows(AlreadyExistsException.class,
                () -> reviewService.create(user1.getId(), movie1.getId(), new ReviewEntity("dsdd", 5)));
    }

    @Test
    void createNotUniqueTest() {
        final ReviewEntity nonUniqueReview = new ReviewEntity("dsdd", 5);
        Assertions.assertThrows(AlreadyExistsException.class, () -> reviewService.create(user1.getId(), movie1.getId(),
                nonUniqueReview));
    }

    @Test
    void createNullableTest() {
        final ReviewEntity nullableReview = new ReviewEntity(null, null);
        Assertions.assertThrows(NotFoundException.class,
                () -> reviewService.create(0L, 0L, nullableReview));
        Assertions.assertThrows(NotFoundException.class,
                () -> reviewService.create(user2.getId(), 0L, nullableReview));
        Assertions.assertThrows(DataIntegrityViolationException.class,
                () -> reviewService.create(user2.getId(), movie2.getId(), nullableReview));
    }

    @Test
    void updateTest() {
        final String test = "TEST";
        final ReviewEntity entity = reviewService.get(user1.getId(), last.getId());
        final String oldDesc = entity.getDescription();
        final ReviewEntity newEntity = reviewService.update(user1.getId(), last.getId(),
                new ReviewEntity(test, 5));
        Assertions.assertEquals(3, reviewService.getAll(0L, 0L).size());
        Assertions.assertEquals(newEntity, reviewService.get(user1.getId(), last.getId()));
        Assertions.assertEquals(test, newEntity.getDescription());
        Assertions.assertNotEquals(oldDesc, newEntity.getDescription());
    }

    @Test
    void deleteTest() {
        reviewService.delete(last.getUser().getId(), last.getId());
        Assertions.assertEquals(2, reviewService.getAll(0L, 0L).size());

        final ReviewEntity newEntity = reviewService.create(user1.getId(), movie2.getId(),
                new ReviewEntity("3210", 5));
        Assertions.assertEquals(3, reviewService.getAll(0L, 0L).size());
        Assertions.assertNotEquals(last.getId(), newEntity.getId());
    }
}
